#!/bin/sh
color_subr="/usr/share/veil/color.subr"
system_subr="/usr/share/veil/system.subr"
deb_subr="/usr/share/veil/deb.subr"

set -e
. ${color_subr}
. ${system_subr}
. ${deb_subr}
set +e

nginx_early()
{
	local _ret=0

	env PAGER=cat ${SYSTEMCTL_CMD} status nginx
	_ret=$?
	[ ${_ret} -eq 0 ] && return 0
	${SYSTEMCTL_CMD} enable nginx
	${SYSTEMCTL_CMD} restart nginx
}

check_hier()
{
	${ECHO} "* ${H5_COLOR}${MY_APPLICATION}: ${H1_COLOR}check_hier${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log

	[ ! -d "${CFG_ETC_DIR}" ] && mkdir -p ${CFG_ETC_DIR}
	[ ! -d "${NODE_DIR}" ] && ${MKDIR_CMD} ${NODE_DIR}
	[ ! -d "${NODE_DIR}/bin" ] && ${MKDIR_CMD} ${NODE_DIR}/bin
	[ ! -d "${PPT_NODE_CONFIG_DIR}" ] && ${MKDIR_CMD} ${PPT_NODE_CONFIG_DIR}
	[ ! -d "${NODE_UEFI_NVRAM_DIR}" ] && ${MKDIR_CMD} ${NODE_UEFI_NVRAM_DIR}
	[ ! -d "${NODE_UEFI_NVRAM_ORIG_DIR}" ] && ${MKDIR_CMD} ${NODE_UEFI_NVRAM_ORIG_DIR}
	[ ! -d "${NODE_DIR}/domains" ] && ${MKDIR_CMD} ${NODE_DIR}/domains

	${CHOWN_CMD} ${NODE_SYS_USER}:${VEIL_GROUP} ${NODE_DIR}/domains
	${CHOWN_CMD} ${NODE_SYS_USER}:${VEIL_GROUP} ${NODE_UEFI_NVRAM_DIR} ${NODE_UEFI_NVRAM_ORIG_DIR}
	${CHMOD_CMD} 0775 ${NODE_UEFI_NVRAM_DIR} ${NODE_UEFI_NVRAM_ORIG_DIR} ${NODE_DIR}/domains

	if [ ! -d /run/gunicorn ]; then
		${MKDIR_CMD} /run/gunicorn
		${CHMOD_CMD} 0775 /run/gunicorn
		${CHOWN_CMD} ${NODE_SYS_USER}:veil /run/gunicorn
	fi

	# secure /root directory
	# Issue #787
	# ворк-эраунд присутствует в сборке ISO. Здесь дублируем только для того, чтобы
	# поправить ситуацию в окружениях, которые обновляются через upgrade
	${CHMOD_CMD} 0700 /root

	# back-compatible (corosync authkey write); technical debt. should go away after >= SpaceVM 6.4
	# keep perms in sync with space_controller::files hier
	if [ ! -d /var/lib/ecp-veil/controller/local_data ]; then
		${MKDIR_CMD} -m 0775 -p /var/lib/ecp-veil/controller/local_data
		${CHOWN_CMD} :veil /var/lib/ecp-veil/controller/local_data
	fi

	return 0
}

trap "/var/lib/ecp-veil/utils/other/spacevm-maintenance -m off" HUP INT ABRT BUS TERM EXIT

[ -r /var/lib/ecp-veil/tests/space-node-first-boot.passed ] && err 0 "${N1_COLOR}already first-booted: ${N2_COLOR}/var/lib/ecp-veil/tests/space-node-first-boot.passed${N0_COLOR}"

if [ ! -r /var/lib/ecp-veil/tests/first_boot.passed ]; then
	maxwait=300
	for i in $( ${SEQ_CMD} 1 ${maxwait} ); do
		if [ ${i} -eq ${maxwait} ]; then
			err 1 "${W1_COLOR}space-utils not ready. ${N1_COLOR}Please checkout /var/log/veil/first_boot.log then re-run: ${N2_COLOR}/var/lib/ecp-veil/node/other/first_boot.sh${N0_COLOR}"
		fi
		${ECHO} "${N1_COLOR}Wating for space-utils first boot (/var/lib/ecp-veil/tests/first_boot.passed): ${N2_COLOR}${i}/${maxwait}${N0_COLOR}"
		sleep 1
		[ -r /var/lib/ecp-veil/tests/first_boot.passed ] && break
	done
fi

# wait for controller passed when we have controller+node installation
# todo: better controller sign
if [ -r /var/lib/ecp-veil/controller/app/requirements.txt ]; then
	# controller + setup
	if [ ! -r /var/lib/ecp-veil/tests/space-controller-first-boot.passed ]; then
		# it might be long
		maxwait=700
		for i in $( ${SEQ_CMD} 1 ${maxwait} ); do
			if [ ${i} -eq ${maxwait} ]; then
				err 1 "${W1_COLOR}space-controller not ready. ${N1_COLOR}Please checkout /var/log/veil/first_boot.log then re-run: ${N2_COLOR}/var/lib/ecp-veil/controller/other/first_boot.sh${N0_COLOR}"
			fi
			${ECHO} "${N1_COLOR}Wating for space-controller first boot (/var/lib/ecp-veil/tests/space-controller-first-boot.passed): ${N2_COLOR}${i}/${maxwait}${N0_COLOR}"
			sleep 1
			[ -r /var/lib/ecp-veil/tests/space-controller-first-boot.passed ] && break
		done
	fi
fi

# обслуживаем maintenance. сервис мог быть запущен на стадии space-utils/space-controller first-boot
nginx_early

# space-utils passed
echo "--" >> /var/log/veil/first_boot.log
${ECHO} "${N1_COLOR}space-node first-boot in progress, logs: ${N2_COLOR}/var/log/veil/first_boot.log${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log

full_st_time=$( ${DATE_CMD} +%s )

# Активация и запуск ntp-wait.service. Предполагается, что это первое включение ntp-wait сервиса (postinst содержит те же строчки для обратной совместимости
# при обновлении с инсталляций, которые не содержали ntp-wait скрипта
# Поскольку first-boot node отрабатывает после first-boot (содержащий применение роли NTP), мы уверены, что будем слать запросы на NTP сервера указанные в IaC.
${SYSTEMCTL_CMD} enable ntp-wait.service || true
${SYSTEMCTL_CMD} start --no-block ntp-wait.service > /dev/null || true

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(61-65))+65 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-node: net_helper"

# networking, legacy
${ECHO} "${N1_COLOR}run net_helper...${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
/var/lib/ecp-veil/utils/other/net_helper.sh | ${TEE_CMD} -a /var/log/veil/first_boot.log

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(65-75))+75 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-node: init_node_db"

# node first boot script wrapper
${ECHO} "${N1_COLOR}space-node-first-boot exec: ${N2_COLOR}/var/lib/ecp-veil/node/other/init_node_db${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
env NOCOLOR=1 /var/lib/ecp-veil/node/other/init_node_db >> /var/log/veil/first_boot.log 2>&1
_ret=$?

[ ${_ret} -ne 0 ] && err 1 "${W1_COLOR}node first boot script failed. ${N1_COLOR}Please checkout /var/log/veil/first_boot.log then re-run: ${N2_COLOR}/var/lib/ecp-veil/node/other/first_boot.sh${N0_COLOR}"

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(76-88))+88 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-node: apply role"

${ECHO} "${N1_COLOR}space-node-first-boot: exec /usr/local/sbin/veil-role -a profile::custom_facts,redis,space_cli,pgsql,pgbouncer,libvirt,fluentd,space_node${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log

# todo: alternative .pid file for puppet role to avoid collision with @reboot !
for i in 1 2 3; do
	/usr/local/sbin/veil-role -a profile::custom_facts,redis,space_cli,pgsql,pgbouncer,libvirt,fluentd,space_node | ${TEE_CMD} -a /var/log/veil/first_boot.log
	_ret=$?
	[ ${_ret} -eq 0 ] && break
done

${ECHO} "${N1_COLOR}space-node-first-boot: exec /root/bin/puppet-apply${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
# full pass
for i in 1 2 3; do
	/root/bin/puppet-apply | ${TEE_CMD} -a /var/log/veil/first_boot.log
	_ret=$?
	[ ${_ret} -eq 0 ] && break
done

PROGRESS=$( ${AWK_CMD} 'BEGIN{srand();print int(rand()*(90-99))+99 }' )
/var/lib/ecp-veil/utils/other/spacevm-maintenance -m on -p ${PROGRESS} -s "SpaceVM first-boot: configure space-node: start services"

# hier
check_hier

if [ ! -r /var/lib/ecp-veil/controller/app/requirements.txt ]; then
	${ECHO} "${N1_COLOR}space-node: reload gunicorn-django-node services...${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log
	# reload daphna/other
	systemctl daemon-reload || true
	stop_services gunicorn-django-node gunicorn-proxy-node gunicorn-uploader-node
	enable_services gunicorn-django-node gunicorn-proxy-node gunicorn-uploader-node
	start_services gunicorn-django-node gunicorn-proxy-node gunicorn-uploader-node
fi

${ECHO} "${N1_COLOR}space-node first-boot complete successfull in: ${N2_COLOR}${full_diff_time}${N0_COLOR}" | ${TEE_CMD} -a /var/log/veil/first_boot.log

# Get packages stats
calc_deb_packages_status

full_end_time=$( ${DATE_CMD} +%s )
full_diff_time=$(( full_end_time - full_st_time ))
# export to metrics
put_prometheus_file_metrics "first_boot" "first_boot_space_node" ${full_diff_time}
full_diff_time=$( displaytime ${full_diff_time} )

# ALL_PACKAGES_NUM and ALL_PACKAGES_BYTES
put_prometheus_file_metrics "first_boot" "first_boot_space_node_all_packages_num" ${ALL_PACKAGES_NUM}
put_prometheus_file_metrics "first_boot" "first_boot_space_node_all_packages_bytes" ${ALL_PACKAGES_BYTES}

# This file is awaiting a CLI and some postinst scripts of other Space Components
${TOUCH_CMD} /var/lib/ecp-veil/tests/space-node-first-boot.passed

/var/lib/ecp-veil/utils/other/spacevm-maintenance -m off

exit ${_ret}
